using System;
using System.Collections;
using System.Collections.Generic;

namespace Esempio1
{
    public class Contenitore : IEnumerable<char>
    {
        //  Carattere iniziale della sequenza
        private readonly char _c;
        //  Numero di caratteri della sequenza
        private readonly int _length;

        public Contenitore(char c, int length)
        {
            if (length < 0)
                throw new ArgumentException("length < 0");
            _c = c;
            _length = length;
        }

        public int Length
        {
            get { return _length; }
        }

        public char this[int index]
        {
            get
            {
                if (index < 0 || index >= _length)
                    throw new IndexOutOfRangeException();
                return (char) (_c + index);
            }
        }

        #region IEnumerable<char> Members

        public IEnumerator<char> GetEnumerator()
        {
            return new Enumeratore(this);
        }

        #endregion

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return new Enumeratore(this);
        }

        #endregion

        private class Enumeratore : IEnumerator<char>
        {
            private readonly Contenitore _contenitore;
            private int _index;

            public Enumeratore(Contenitore contenitore)
            {
                _contenitore = contenitore;
                Reset();
            }

            #region IEnumerator<char> Members

            public char Current
            {
                get
                {
                    if (_index == -1 || _index >= _contenitore.Length)
                        throw new InvalidOperationException();
                    return _contenitore[_index];
                }
            }

            #endregion

            #region IEnumerator Members

            object IEnumerator.Current
            {
                get { return Current; }
            }

            public bool MoveNext()
            {
                if (_index < _contenitore.Length)
                    _index++;
                return (_index < _contenitore.Length);
            }

            public void Reset()
            {
                _index = -1;
            }

            #endregion

            #region IDisposable Members

            public void Dispose()
            {
            }

            #endregion
        }
    }
}
