using System;

namespace Esempio1
{
    public struct Digit
    {
        private readonly byte _value;  //  0..9

        public Digit(byte value)
        {
            if (value > 9)
                throw new ArgumentException("value > 9");
            _value = value;
        }

        public Digit(int value)
            : this(Convert.ToByte(value))
        {
        }

        public byte Value
        {
            get { return _value; }
        }

        #region Conversioni

        //  Digit y = new Digit(5);
        //  Invece di scrivere:
        //  byte x = y.Value;
        //  posso scrivere:
        //  byte x = y;
        public static implicit operator byte(Digit digit)
        {
            return digit.Value;
        }

        //  Invece di scrivere:
        //  Digit y = new Digit(5);
        //  posso scrivere:
        //  Digit y = (Digit) 5;
        public static explicit operator Digit(byte value)
        {
            return new Digit(value);
        }

        public static explicit operator Digit(int value)
        {
            return new Digit(value);
        }

        public static implicit operator char(Digit digit)
        {
            return (char) (digit.Value + '0');
        }

        public static explicit operator Digit(char value)
        {
            return new Digit(value - '0');
        }

        #endregion

        public override string ToString()
        {
            return ((char) this).ToString();
        }
    }
}
